import 'package:string_sequence/alphabet.dart';

/// An [Alphabet] implementation whose sequence of symbols is repeated forever.
class LoopingAlphabet extends AlphabetTransformer {
  LoopingAlphabet(Alphabet alphabet) : super(alphabet);

  factory LoopingAlphabet.fromSet(Set<String> fromSet) =>
      new LoopingAlphabet(new Alphabet.fromSet(fromSet));

  /// Returns the symbol after [char] in this alphabet. If char == [end] then
  /// [start] is returned.
  @override
  String successor(String char) => super.successor(char) ?? start;

  /// Returns the symbol before [char] in this alphabet. If char == [start] then
  /// [end] is returned.
  @override
  String predecessor(String char) => super.predecessor(char) ?? end;
}

/// The letters of the English [Alphabet] in their usual order.
class EnglishAlphabet implements Alphabet {
  final String _start;
  final String _end;

  EnglishAlphabet()
      : _start = 'a',
        _end = 'z';

  EnglishAlphabet.uppercase()
      : _start = 'A',
        _end = 'Z';

  @override
  String get start => _start;

  @override
  String get end => _end;

  @override
  bool contains(String char) =>
      char.codeUnitAt(0) >= start.codeUnitAt(0) &&
      char.codeUnitAt(0) <= end.codeUnitAt(0);

  @override
  String successor([String char]) {
    if (char == end || !contains(char)) return null;
    return new String.fromCharCode(char.codeUnitAt(0) + 1);
  }

  @override
  String predecessor(String char) {
    if (char == start || !contains(char)) return null;
    return new String.fromCharCode(char.codeUnitAt(0) - 1);
  }
}
