import 'package:string_sequence/common_alphabets.dart';
import 'package:string_sequence/enumerating_string_sequence.dart';
import 'package:test/test.dart';

void main() {
  group('$EnumeratingStringSequence', () {
    group('next', () {
      EnumeratingStringSequence sequence;

      void commonSetUp() {
        sequence = new EnumeratingStringSequence(
            new LoopingAlphabet.fromSet(['A', 'B', 'C'].toSet()));
      }

      test('should return the given alphabet\s start symbol on the first call.',
          () {
        commonSetUp();
        expect(sequence.next(), 'A');
      });

      test('should initially iterate over all symbols in the given alphabet',
          () {
        commonSetUp();
        expect(sequence.next(), 'A');
        expect(sequence.next(), 'B');
        expect(sequence.next(), 'C');
      });

      test(
          'should increment previous characters when the last symbol in a '
          'string loops to the start of the given alphabet.', () {
        commonSetUp();
        sequence..next()..next()..next(); // A, B, C
        expect(sequence.next(), 'AA');

        sequence..next()..next(); // AB, AC
        expect(sequence.next(), 'BA');
      });

      test(
          'should increment previous characters when a symbol in the middle of '
          'a string loops to the start of alphabet.', () {
        commonSetUp();
        var sequence = new EnumeratingStringSequence.startingAfter(
            'BCC', new LoopingAlphabet.fromSet(['A', 'B', 'C'].toSet()));
        expect(sequence.next(), 'CAA');
      });
    });
  });
}
